/*************************************************
* X.509 CRL Header File                          *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#ifndef BOTAN_X509_CRL_H__
#define BOTAN_X509_CRL_H__

#include <botan/asn1_obj.h>
#include <vector>

namespace Botan {

/*************************************************
* X.509 CRL                                      *
*************************************************/
class X509_CRL
   {
   public:
      struct X509_CRL_Error : public Exception
         {
         X509_CRL_Error(const std::string& error) :
            Exception("X509_CRL: " + error) {}
         };

      struct CRL_Info
         {
         public:
            SecureVector<byte> serial;
            X509_Time time;
            CRL_Code reason;
         };

      std::vector<CRL_Info> get_revoked() const;

      X509_DN issuer_dn() const;
      SecureVector<byte> authority_key_id() const;

      SecureVector<byte> tbs_certlist() const;
      SecureVector<byte> signature() const;
      AlgorithmIdentifier signature_algorithm() const;

      u32bit crl_number() const;
      X509_Time this_update() const;
      X509_Time next_update() const;

      X509_CRL(DataSource&, X509_Encoding = PEM);
   private:
      void decode_crl(DataSource&);
      void handle_crl_entry_extension(CRL_Info&, const Extension&);
      void handle_crl_extension(const Extension&);
      std::vector<CRL_Info> revoked;
      X509_DN issuer;
      X509_Time start, end;
      SecureVector<byte> issuer_key_id, tbs_crl_data, sig;
      AlgorithmIdentifier sig_algo;
      u32bit version, crl_count;
   };

}

#endif
